<?php

namespace app\Controllers;

use app\Controllers\Controller;
use app\Core\Request;
use app\Core\Response;
use app\Forms\Email;
use app\Forms\Field;
use app\Forms\Minimum;
use app\Forms\Phone;
use app\Forms\Required;
use app\Forms\Validation;
use app\Models\User;
use app\Utils\Toast;

class AuthController extends Controller
{


  public function login()
  {
    Response::$title = "Login | MyKos";
    $form = new Validation([
      [new Field('email'), [new Required()]],
      [new Field('password'), [new Required()]]
    ]);
    if (Request::isAuth()) {
      Response::redirect('/');
      return;
    }
    if (Request::getMethod() === "POST") {
      $data = Request::getBody();
      if (!$form->validate($data)) {
        $this->render('login', $form->getFields());
        return;
      }

      $user = User::whereFirst(['email' => $data['email'] ?? '']);

      if (!$user || !password_verify($data['password'], $user->password)) {
        $form->addError('password', 'Email atau password salah');
        $this->render('login', $form->getFields());
        return;
      }

      $_SESSION['user'] = $user->user_id;
      Toast::success("Selamat datang {$user->name}!", true);

      Response::redirect('/');
      return;
    }


    $this->render('login', $form->getFields());
  }

  public function register()
  {
    Response::$title = "Daftar | MyKos";
    $form = new Validation([
      [new Field('email'), [new Required(), new Email()]],
      [new Field('name', "Nama"), [new Required()]],
      [new Field('phone_number', "No Telepon"), [new Phone()]],
      [new Field('password'), [new Required(), new Minimum(4)]],
      [new Field('password2', "Konfirmasi Password"), [new Required()]],
    ]);
    if (Request::isAuth()) {
      Response::redirect('/');
      return;
    }

    if (Request::getMethod() == "POST") {
      $valid = $form->validate(Request::getBody());
      if ($form->data["password"] !== $form->data["password2"]) {
        $valid = false;
        $form->addError('password2', 'Password tidak sama');
      }
      if (!$valid) {
        $this->render('register', $form->getFields());
        return;
      }

      $user = User::whereFirst(['email' => $form->data['email'] ?? '']);

      if ($user) {
        $form->addError('email', 'Email sudah terdaftar');
        $this->render('register', $form->getFields());
        return;
      }

      $user = User::toModel($form->data);

      $user->password = password_hash($user->password, PASSWORD_DEFAULT);

      $user->save();
      $_SESSION['user'] = $user->user_id;
      Toast::success("Selamat datang {$user->name}!", true);
      Response::redirect('/');
      return;
    }

    $this->render('register', $form->getFields());
  }

  public function logout()
  {
    unset($_SESSION['user']);
    Response::redirect('/login');
  }

  public function me()
  {
    var_dump(Request::getUser());
  }

  public function test($param)
  {
    var_dump($param);
  }
}
