<?php

namespace app\Controllers;

use app\Core\Request;
use app\Core\Response;
use app\Utils\Toast;

class Controller
{
  private static $head = '';
  protected static function render($view, $data = [])
  {
    $data["user"] = Request::getUser();
    $data["__head"] = self::$head;
    $data["__title"] = Response::$title ?? "MyKos";
    if (Toast::$toast) {
      $data["toast"] = Toast::$toast;
      Toast::$toast = null;
    }
    echo self::renderView($view, $data);
  }

  private static function renderView($view, &$data = [])
  {
    extract($data);
    ob_start();
    include_once __DIR__ . "/../Views/$view.php";
    $content = ob_get_clean();
    $parentRegex = "/^@extends\('(.*)'\)/";
    $headRegex = "/@@head([\s\S]*)@@endhead/";
    if (preg_match($headRegex, $content, $matches)) {
      self::$head = $matches[1] . self::$head;
      $data["__head"] = self::$head;
      $content = preg_replace($headRegex, "", $content);
    }
    if (preg_match($parentRegex, $content, $matches)) {
      $parentView = $matches[1];
      $data["__content"] = preg_replace($parentRegex, "", $content);
      return self::renderView($parentView, $data);
    } else {
      return $content;
    }
  }

  public static function notFound()
  {
    Response::statusCode(404);
    self::render("404");
  }

  public static function methodNotAllowed()
  {
    Response::statusCode(405);
    self::render("405");
  }
}
