<?php


namespace app\Controllers;

use app\Core\Request;
use app\Core\Response;
use app\Forms\Field;
use app\Forms\Number;
use app\Forms\Phone;
use app\Forms\Required;
use app\Forms\Validation;
use app\Models\Owners;
use app\Utils\Toast;

class OwnerController extends Controller
{
  public function __construct()
  {
    Response::$title = "Daftar Pemilik | MyKos";
  }

  public function index()
  {
    $form = new Validation([
      [new Field('name', "Nama"), [new Required()]],
      [new Field('phone_number', "Nomor Telepon"), [new Required(), new Phone()]]
    ]);
    if (Request::getMethod() === "POST") {
      if ($form->validate(Request::getBody())) {
        $owner = Owners::toModel($form->data);
        $owner->save();
        Toast::success("Pemilik berhasil ditambahkan", true);
        Response::redirect("/owners");
        return;
      }
    }
    $owners = Owners::all("owner_id ASC");
    $this->render("owner/index", [
      "owners" => $owners,
      ...$form->getFields(),
    ]);
  }

  public function update()
  {
    $form = new Validation([
      [new Field('name', "Nama"), [new Required()]],
      [new Field('phone_number', "Nomor Telepon"), [new Required(), new Phone()]],
      [new Field('owner_id'), [new Number()]]
    ]);

    if ($form->validate(Request::getBody())) {
      $owner = Owners::toModel($form->data);
      $owner->save();
      Toast::success("Pemilik berhasil diubah", true);
      Response::redirect("/owners");
    } else {
      $owners = Owners::all("owner_id ASC");
      $this->render("owner/index", [
        "owners" => $owners,
        "editId" => $form->data["owner_id"],
        ...$form->getFields(),
      ]);
    }
  }

  public function delete()
  {
    $form = new Validation([
      [new Field('owner_id'), [new Number()]]
    ]);
    if ($form->validate(Request::getBody())) {
      try {

        Owners::deleteById($form->data["owner_id"]);
      } catch (\PDOException $e) {
        // foreign key constraint violation
        if ($e->getCode() === "23503") {
          Toast::error("Pemilik tidak dapat dihapus karena masih memiliki kos");
          $this->render("owner/index", [
            "owners" => Owners::all("owner_id ASC"),
          ]);
          return;
        }
        throw $e;
      }
    }
    Toast::success("Pemilik berhasil dihapus", true);
    Response::redirect("/owners");
  }
}
