<?php


namespace app\Controllers;

use app\Core\Request;
use app\Core\Response;
use app\Forms\Field;
use app\Forms\Phone;
use app\Forms\Email;
use app\Forms\Required;
use app\Forms\Validation;
use app\Forms\Minimum;
use app\Forms\Number;
use app\Forms\Rating;
use app\Models\Owners;
use app\Models\Reviews;
use app\Models\User;
use app\Utils\Toast;


class ReviewController extends Controller
{
  public function __construct()
  {
    Response::$title = "Review | MyKos";
  }

  public function getAllByDormId($params)
  {
    $dorm = $params['dormId'];
    $reviews = Reviews::findByDormId($dorm->dorm_id);
    $this->render("reviews/edit", [
      "dorm_id" => $dorm->dorm_id,
      "reviews" => $reviews
    ]);
  }

  public function create($params)
  {
    $form = $this->getReviewForm();
    $dormId = $params['dormId'];


    if (Request::getMethod() === "POST") {
      if ($form->validate(Request::getBody())) {
        $review = Reviews::toModel($form->data);
        $review->user_id = Request::getUser()->user_id;
        $review->dorm_id = $dormId;
        $review->save();
        Toast::success("Review berhasil ditambahkan", true);
        Response::redirect("/dorms/{$dormId}");
        return;
      }
    }
    $userId = Request::getUser()->user_id;
    $this->render("review/create", [
      "userId" => $userId,
      ...$form->getFields()
    ]);
  }

  public function getReviewForm()
  {
    return new Validation([
      [new Field("rate", "Rating"), [new Required(), new Rating()]],
      [new Field("description", "Deskripsi"), [new Required()]],
    ]);
  }

  private function setReviewFormData(Validation $form, Reviews $review)
  {
    $form->data["rate"] = $review->rate;
    $form->data["description"] = $review->description;
  }

  public function edit($params)
  {
    $reviewId = $params["reviewId"];
    $review = Reviews::findById($reviewId);
    $userId = $review->user_id;
    $dormId = $review->dorm_id;
    $formReview = $this->getReviewForm();

    if (Request::getMethod() === "POST") {
      if ($formReview->validate(Request::getBody())) {
        $review = Reviews::toModel($formReview->data);
        $review->review_id = $reviewId;
        $review->user_id = $userId;
        $review->dorm_id = $dormId;
        $review->save();
        Toast::success("Review berhasil diubah", true);
        Response::redirect("/dorms/{$dormId}");
        return;
      }
    } else {
      if (Request::getUser()->user_id !== $userId) {
        Response::redirect("/dorms/{$dormId}");
        return;
      }
      $this->setReviewFormData($formReview, $review);
    }

    $user = Request::getUser();
    $this->render("review/create", [
      "reviewId" => $reviewId,
      ...$formReview->getFields()
    ]);
  }

  public function deleteById($params)
  {
    $dormId = $params["dormId"];

    $form = new Validation([
      [new Field('review_id'), [new Number()]]
    ]);

    if ($form->validate(Request::getBody())) {
      $reviewId = $form->data["review_id"];
      Reviews::deleteById($reviewId);
    }

    Toast::success("Review berhasil dihapus", true);
    Response::redirect("/dorms/{$dormId}");
  }
}
