<?php


namespace app\Controllers;

use app\Core\Request;
use app\Core\Response;
use app\Forms\Field;
use app\Forms\Phone;
use app\Forms\Email;
use app\Forms\Required;
use app\Forms\Validation;
use app\Forms\Minimum;
use app\Models\Owners;
use app\Models\User;
use app\Utils\Toast;


class UserController extends Controller
{
  public function __construct()
  {
    Response::$title = "Akun | MyKos";
  }

  //   public function me()
  //   {
  //     $form = new Validation([
  //       [new Field('email', "Email"), [new Required(), new Email()]],
  //       [new Field('name', "Nama"), [new Required()]],
  //       [new Field('phone_number', "Nomor Telepon"), [new Required(), new Phone()]],
  //     //   [new Field('password'), [new Required()]]
  //     ]);
  //     if (Request::getMethod() === "POST") {
  //       if ($form->validate(Request::getBody())) {
  //         $user = User::toModel($form->data);
  //         $user->save();
  //         Toast::success("Penyewa berhasil ditambahkan", true);
  //         Response::redirect("/owners");
  //         return;
  //       }
  //     }
  //     $user = Request::getUser();
  //     $this->render("user/index", [
  //       "user" => $user,
  //       ...$form->getFields(),
  //     ]);
  //   }

  public function edit()
  {
    $userId = Request::getUser()->user_id;
    $formProfile = $this->getProfileForm();

    if (Request::getMethod() === "POST") {
      if ($formProfile->validate(Request::getBody())) {
        $user = User::toModel($formProfile->data);
        $user->user_id = $userId;
        $user->save();
        Toast::success("Profil berhasil diubah", true);
        Response::redirect("/account/edit");
        return;
      }
    } else {
      $user = User::findById($userId);
      $this->setUserFormData($formProfile, $user);
    }

    $user = Request::getUser();
    $this->render("user/edit", [
      "user" => $user,
      ...$formProfile->getFields(),
    ]);
  }

  public function editPassword()
  {
    $user = Request::getUser();
    $formProfile = $this->getProfileForm();
    $formPassword = $this->getPasswordForm();

    $this->setUserFormData($formProfile, $user);

    if (Request::getMethod() === "POST") {
      $valid = $formPassword->validate(Request::getBody());
      $data = $formPassword->data;
      if (!password_verify($data['password'], $user->password)) {
        $formPassword->addError('password', 'Password salah');
        $valid = false;
      }
      if ($data["password1"] !== $data["password2"]) {
        $formPassword->addError('password2', 'Password tidak sama');
        $valid = false;
      }
      if (!$valid) {
        $this->setUserFormData($formProfile, $user);

        $formPassword->data["password"] = $data["password"];
        $formPassword->data["password1"] = $data["password1"];
        $formPassword->data["password2"] = $data["password2"];

        $this->render('user/edit', [
          "user" => $user,
          "fields" => $formProfile->data + $formPassword->data,
          "errors" => $formProfile->errors + $formPassword->errors
        ]);
        return;
      }

      $userId = Request::getUser()->user_id;
      $user = User::findById($userId);
      $user->user_id = $userId;
      $user->password = password_hash($data["password1"], PASSWORD_DEFAULT);

      $user->save();
      Toast::success("Password berhasil diubah", true);
      Response::redirect("/account/edit");
      return;
    }
  }


  private function setUserFormData(Validation $form, User $user)
  {
    $form->data["name"] = $user->name;
    $form->data["email"] = $user->email;
    $form->data["phone_number"] = $user->phone_number;
  }

  //   private function setPasswordFormData(Validation $form, User $user)
  //   {
  //     $form->data["password"] = $user->name;
  //     $form->data["password1"] = $user->email;
  //     $form->data["password2"] = $user->phone_number;
  //   }


  private function getProfileForm()
  {
    return new Validation([
      [new Field("name", "Nama"), [new Required()]],
      [new Field("email", "Email"), [new Required(), new Email()]],
      [new Field("phone_number", "Nomor Telepon"), [new Required(), new Phone()]],
    ]);
  }

  private function getPasswordForm()
  {
    return new Validation([
      [new Field('password', "Password Lama"), [new Required()]],
      [new Field('password1', "Passwor Baru"), [new Required(), new Minimum(4)]],
      [new Field('password2', "Konfirmasi Password"), [new Required()]]
    ]);
  }
}
