<?php

namespace app\Core;

use app\Controllers\OwnerController;
use app\Controllers\AuthController;
use app\Controllers\DormController;
use app\Controllers\UserController;
use app\Controllers\ReviewController;
use app\Controllers\WishlistController;
use app\Middlewares\AdminOnly;
use app\Middlewares\AuthRequired;
use app\Models\BaseModel;
use app\Utils\Toast;

class Application
{
  public Router $router;
  public static Database $db;

  public function __construct()
  {
    $this->configureRoutes();
    self::$db = new Database();
    self::$db->connect();
    BaseModel::connect();
    Toast::initialize();
  }

  public function run()
  {
    $this->router->dispatch();
  }

  public function configureRoutes()
  {
    $this->router = new Router();
    $this->router->get('/', [], DormController::class, 'list');
    $this->router->get('/dorms', [], DormController::class, 'list');
    $this->router->methods(["GET", "POST"], '/login', [],  AuthController::class, 'login');
    $this->router->methods(["GET", "POST"], '/register', [],  AuthController::class, 'register');
    $this->router->get('/logout', [],  AuthController::class, 'logout');

    $this->router->methods(["GET", "POST"], '/owners', [AdminOnly::class], OwnerController::class, 'index');
    $this->router->put("/owners", [AdminOnly::class], OwnerController::class, 'update');
    $this->router->delete("/owners", [AdminOnly::class], OwnerController::class, 'delete');
    $this->router->get('/users/{user_id}/{dorm_id}', [],  AuthController::class, 'test');
    $this->router->get("/me", [AuthRequired::class], AuthController::class, 'me');

    $this->router->methods(["GET", "POST"], "/dorms/create", [AdminOnly::class], DormController::class, 'create');
    $this->router->methods(["GET", "POST"], "/dorms/{dormId}", [], DormController::class, 'view');
    $this->router->delete("/dorms/{dormId}", [AuthRequired::class], DormController::class, 'delete');
    $this->router->methods(["GET", "POST"], "/dorms/{dormId}/media", [AdminOnly::class], DormController::class, 'media');
    $this->router->delete("/dorms/{dormId}/media", [AdminOnly::class], DormController::class, 'deleteMedia');
    $this->router->methods(["GET", "POST"], "/dorms/{dormId}/edit", [AdminOnly::class], DormController::class, 'edit');
    $this->router->methods(["GET", "POST"], "/dorms/{dormId}/reviews", [AuthRequired::class], ReviewController::class, 'getAllByDormId');
    $this->router->methods(["GET", "POST"], "/dorms/{dormId}/add-review", [AuthRequired::class], ReviewController::class, 'create');

    $this->router->methods(["GET", "POST"], "/account/edit", [AuthRequired::class],  UserController::class, 'edit');
    $this->router->methods(["GET", "POST"], "/account/editPassword", [AuthRequired::class],  UserController::class, 'editPassword');

    $this->router->methods(["GET", "POST"], "/reviews/{reviewId}", [AuthRequired::class], ReviewController::class, 'edit');
    $this->router->delete("/dorms/{dormId}/delete-review", [AuthRequired::class], ReviewController::class, 'deleteById');

    $this->router->methods(["GET"], "/wishlist", [AuthRequired::class], WishlistController::class, 'index');
    $this->router->methods(["GET"], "/dorms/{dormId}/add-to-wishlist", [AuthRequired::class], WishlistController::class, 'add');
    $this->router->delete("/wishlist", [AuthRequired::class], WishlistController::class, 'delete');
  }
}
