<?php

namespace app\Core;

use app\Models\User;

class Request
{
  private static ?string $path = null;
  private static ?User $user;

  public static function getPath(): string
  {
    if (self::$path !== null) {
      return self::$path;
    }
    $path = $_SERVER['REQUEST_URI'] ?? '/';
    $qIndex = strpos($path, '?');
    if ($qIndex === false) {
      return $path;
    }
    self::$path = substr($path, 0, $qIndex);
    return self::$path;
  }

  public static function getMethod()
  {
    $realMethod = $_SERVER['REQUEST_METHOD'];
    if ($realMethod === "POST" && isset($_POST['_method']) && in_array($_POST['_method'], ["PUT", "DELETE"])) {
      return $_POST['_method'];
    }
    return $realMethod;
  }

  public static function getBody()
  {
    return $_POST;
  }

  public static function getUser()
  {
    if (isset(self::$user)) {
      return self::$user;
    }
    if (!isset($_SESSION['user'])) {
      self::$user = null;
      return null;
    }
    self::$user = User::findById($_SESSION['user']);
    return self::$user;
  }

  public static function isAuth()
  {
    return isset($_SESSION['user']);
  }

  public static function getFiles($fieldName)
  {
    return $_FILES[$fieldName] ?? null;
  }

  public static function getPage()
  {
    $page = $_GET['page'] ?? "1";
    try {
      return intval($page);
    } catch (\Throwable $th) {
      return 1;
    }
  }

  public static function getQuery()
  {
    return $_GET;
  }

  public static function isJSON()
  {
    return isset($_SERVER["ACCEPT"]) && $_SERVER["ACCEPT"] === "application/json";
  }
}
