<?php

namespace app\Models;

use app\Core\Application;
use app\Core\Database;
use PDO;

#[\AllowDynamicProperties]
class BaseModel
{
  protected static PDO $db;
  protected static string $table = "";
  protected static $primaryKey = "";

  public static function connect()
  {
    self::$db = Application::$db->pdo;
  }

  public static function toModel($data): self
  {
    $model = new static();
    foreach ($data as $key => $value) {
      if (property_exists($model, $key)) {
        $model->$key = $value;
      }
    }
    return $model;
  }

  public static function toModelArray($data)
  {
    $models = [];
    foreach ($data as $row) {
      $models[] = self::toModel($row);
    }
    return $models;
  }

  public static function all($orderBy = null)
  {
    $stmt = self::$db->prepare("SELECT * FROM " . static::$table . ($orderBy ? " ORDER BY " . $orderBy : ""));
    $stmt->execute();
    return self::toModelArray($stmt->fetchAll());
  }

  public static function findById(int $id): static
  {
    $stmt = self::$db->prepare("SELECT * FROM " . static::$table . " WHERE " . static::$primaryKey . " = :id");
    $stmt->bindValue(":id", $id);
    $stmt->execute();
    $data = $stmt->fetch();
    if (!$data) {
      return null;
    }
    return self::toModel($data);
  }

  public static function deleteById(int $id)
  {
    $stmt = self::$db->prepare("DELETE FROM " . static::$table . " WHERE " . static::$primaryKey . " = :id");
    $stmt->bindValue(":id", $id);
    $stmt->execute();
    return $stmt->rowCount();
  }

  public static function where(array $arrFilter)
  {

    $where = "";
    foreach ($arrFilter as $key => $value) {
      $where .= $key . " = :" . $key . " AND ";
    }
    $where = substr($where, 0, -5);

    $stmt = self::$db->prepare("SELECT * FROM " . static::$table . " WHERE " . $where);
    foreach ($arrFilter as $key => $value) {
      $stmt->bindValue(":" . $key, $value);
    }
    $stmt->execute();
    return self::toModelArray($stmt->fetchAll());
  }

  public static function whereFirst(array $arrFilter)
  {

    $where = "";
    foreach ($arrFilter as $key => $value) {
      $where .= $key . " = :" . $key . " AND ";
    }
    $where = substr($where, 0, -5);

    $stmt = self::$db->prepare("SELECT * FROM " . static::$table . " WHERE " . $where . " LIMIT 1");
    foreach ($arrFilter as $key => $value) {
      $stmt->bindValue(":" . $key, $value);
    }
    $stmt->execute();
    $data = $stmt->fetch();
    if (!$data) {
      return null;
    }
    return self::toModel($data);
  }

  public function save()
  {
    if (!is_array(static::$primaryKey) && isset($this->{static::$primaryKey})) {
      $this->update();
    } else {
      $this->insert();
    }
  }

  private function insert()
  {
    $fields = get_object_vars($this);
    if (!is_array(static::$primaryKey)) {
      $fields = array_filter($fields, function ($key) {
        return $key != static::$primaryKey;
      }, ARRAY_FILTER_USE_KEY);
    }

    $keys = array_keys($fields);


    if (is_array(static::$primaryKey)) {
      $sql = "INSERT INTO " . static::$table . " (" . implode(",", $keys) . ") VALUES (:" . implode(",:", $keys) . ") RETURNING " . implode(",", static::$primaryKey) . ";";
    } else {
      $sql = "INSERT INTO " . static::$table . " (" . implode(",", $keys) . ") VALUES (:" . implode(",:", $keys) . ") RETURNING " . static::$primaryKey . ";";
    }

    $stmt = self::$db->prepare($sql);

    foreach ($fields as $key => $value) {
      $stmt->bindValue(":" . $key, $value);
    }

    $stmt->execute();
    $data = $stmt->fetch();
    if (!is_array(static::$primaryKey)) {
      $this->{static::$primaryKey} = $data[static::$primaryKey];
    }
  }

  private function update()
  {
    $fields = get_object_vars($this);
    $fields = array_filter($fields, function ($key) {
      return $key != static::$primaryKey;
    }, ARRAY_FILTER_USE_KEY);

    $keys = array_keys($fields);

    $sql = "UPDATE " . static::$table . " SET ";
    foreach ($keys as $key) {
      $sql .= $key . " = :" . $key . ", ";
    }
    $sql = substr($sql, 0, -2);
    $sql .= " WHERE " . static::$primaryKey . " = :" . static::$primaryKey . ";";

    $stmt = self::$db->prepare($sql);

    foreach ($fields as $key => $value) {
      // Special handling for the 'is_admin' field
      if ($key === 'is_admin') {
        // Convert the boolean value to a string 'true' or 'false'
        $value = $value ? 'true' : 'false';
      }
      $stmt->bindValue(":" . $key, $value);
    }
    $stmt->bindValue(":" . static::$primaryKey, $this->{static::$primaryKey});

    $stmt->execute();
  }

  public function delete()
  {
    $sql = "DELETE FROM " . static::$table . " WHERE " . static::$primaryKey . " = :" . static::$primaryKey . ";";

    $stmt = self::$db->prepare($sql);

    $stmt->bindValue(":" . static::$primaryKey, $this->{static::$primaryKey});

    $stmt->execute();
  }
}
