<?php

namespace app\Models;

use PDO;

class Dorm extends BaseModel
{
  public int $dorm_id;
  public string $name;
  public string $address;
  public string $city;
  public int $price;
  public string $description;
  public int $owner_id;
  public ?float $rate;
  protected static string $table = "dorms";
  protected static $primaryKey = "dorm_id";

  public static function getAllFiltered($page = 1, $orderBy = "dorm_id DESC", $search = "", $city = "", $price = "")
  {
    $limit = 12;
    $offset = ($page - 1) * $limit;
    $cityQuery = $city ? "AND city = '$city'" : "";
    $priceQuery = $price ? "AND $price" : "";
    $rateWhereQuery = $orderBy === "rate" ? "AND rate IS NOT NULL" : "";
    if ($orderBy === "rate") $orderBy = "rate DESC";

    $ratingQuery = "NATURAL LEFT JOIN (SELECT dorm_id, AVG(rate) AS rate FROM reviews GROUP BY dorm_id) AS r";

    $stmt = self::$db->prepare(
      $search ?
        "SELECT * FROM dorms $ratingQuery WHERE (name ILIKE :1 OR city ILIKE :1) $cityQuery $priceQuery $rateWhereQuery ORDER BY $orderBy LIMIT :2 OFFSET :3" :
        "SELECT * FROM dorms $ratingQuery WHERE TRUE $cityQuery $priceQuery $rateWhereQuery ORDER BY $orderBy LIMIT :2 OFFSET :3"
    );

    if ($search) {
      $stmt->bindValue(":1", "%{$search}%");
    }
    $stmt->bindValue(":2", $limit);
    $stmt->bindValue(":3", $offset);
    // echo $stmt->queryString;
    // exit();

    $stmt->execute();
    $dorms = self::toModelArray($stmt->fetchAll());

    $stmt = self::$db->prepare(
      $search ?
        "SELECT COUNT(dorm_id) FROM dorms $ratingQuery WHERE (name ILIKE :1 OR city ILIKE :1) $cityQuery $priceQuery $rateWhereQuery" :
        "SELECT COUNT(dorm_id) FROM dorms $ratingQuery WHERE TRUE $cityQuery $priceQuery $rateWhereQuery"
    );

    if ($search) {
      $stmt->bindValue(":1", "%{$search}%");
    }

    $stmt->execute();

    $count = $stmt->fetchColumn();
    $totalPage = ceil($count / $limit);

    if (count($dorms) === 0) {
      return ["dorms" => [], "medias" => [], "totalPage" => 1];
    }

    $medias = static::getMedias($dorms);

    return ["dorms" => $dorms, "medias" => $medias, "totalPage" => $totalPage];
  }

  public static function getCities()
  {
    $stmt = self::$db->prepare("SELECT DISTINCT city FROM dorms ORDER BY city ASC");
    $stmt->execute();
    return $stmt->fetchAll(PDO::FETCH_COLUMN);
  }

  public static function getInIds(array $ids)
  {
    if (count($ids) == 0) {
      $dorms = array();
      $medias = array();
      return ["dorms" => $dorms, "medias" => $medias];
    }
    $in  = str_repeat('?,', count($ids) - 1) . '?';
    $stmt = self::$db->prepare("SELECT * FROM dorms WHERE dorm_id IN ($in)");
    $stmt->execute($ids);

    $dorms =  self::toModelArray($stmt->fetchAll());
    $medias = static::getMedias($dorms);

    return ["dorms" => $dorms, "medias" => $medias];
  }

  private static function getMedias(array $dorms)
  {
    if (count($dorms) == 0) return array();
    $dormIds = array_map(fn ($dorm) => $dorm->dorm_id, $dorms);

    $stmt = self::$db->prepare("SELECT * FROM medias
      NATURAL JOIN (
        SELECT MIN(media_id) AS media_id, dorm_id FROM medias WHERE type = 'photo' AND dorm_id IN (" . implode(",", $dormIds) . ") GROUP BY dorm_id
        ) AS m");
    $stmt->execute();

    $medias = Media::toModelArray($stmt->fetchAll());
    $medias = array_reduce($medias, function ($acc, $media) {
      $acc[$media->dorm_id] = $media;
      return $acc;
    }, []);
    return $medias;
  }
}
