CREATE TABLE IF NOT EXISTS users (
    user_id     SERIAL NOT NULL,
    email       TEXT NOT NULL UNIQUE,
    name        TEXT NOT NULL,
    password    TEXT NOT NULL,
    phone_number TEXT,
    is_admin     BOOLEAN NOT NULL DEFAULT FALSE,
    created_at   TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
    PRIMARY KEY (user_id)
);

CREATE TABLE IF NOT EXISTS owners (
    owner_id     SERIAL NOT NULL,
    name        TEXT NOT NULL,
    phone_number TEXT,
    created_at   TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
    PRIMARY KEY (owner_id)
);

CREATE TABLE IF NOT EXISTS dorms (
    dorm_id      SERIAL NOT NULL,
    owner_id    INT NOT NULL,
    name        TEXT NOT NULL,
    address     TEXT NOT NULL,
    city        TEXT NOT NULL,
    price       INT NOT NULL,
    description TEXT NOT NULL,
    created_at  TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
    PRIMARY KEY (dorm_id),
    FOREIGN KEY (owner_id) REFERENCES owners(owner_id)
);

--create types
DO $$
BEGIN
    IF NOT EXISTS (SELECT 1 FROM pg_type WHERE typname = 'media_type') THEN
       CREATE TYPE media_type AS ENUM ('photo', 'video');
    END IF;
END$$;

CREATE TABLE IF NOT EXISTS medias (
    media_id    SERIAL NOT NULL,
    dorm_id      INT NOT NULL,
    type        MEDIA_TYPE NOT NULL,
    endpoint    TEXT NOT NULL,
    alt_text    TEXT NOT NULL,
    created_at  TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
    PRIMARY KEY (media_id),
    FOREIGN KEY (dorm_id) REFERENCES dorms(dorm_id) ON DELETE CASCADE
);

CREATE TABLE IF NOT EXISTS reviews (
    review_id   SERIAL NOT NULL,
    dorm_id      INT NOT NULL,
    user_id     INT NOT NULL, 
    rate        SMALLINT NOT NULL,
    description TEXT NOT NULL,
    created_at  TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
    PRIMARY KEY (review_id),
    FOREIGN KEY (dorm_id) REFERENCES dorms(dorm_id) ON DELETE CASCADE,
    FOREIGN KEY (user_id) REFERENCES users(user_id)
);

CREATE TABLE IF NOT EXISTS wishlists (
    user_id     INT NOT NULL,
    dorm_id      INT NOT NULL,
    created_at   TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
    PRIMARY KEY (user_id, dorm_id),
    FOREIGN KEY (user_id) REFERENCES users(user_id),
    FOREIGN KEY (dorm_id) REFERENCES dorms(dorm_id) ON DELETE CASCADE
);

INSERT INTO users (email, name, password, phone_number, is_admin) 
VALUES (
    'admin@gmail.com', 'Admin', '$2y$10$ja.jFnbDWv9SVI880yyfEupt4LgSJFQXNJwkXEBDaApt9gB8pkLvm', '08123456789', TRUE
    ) ON CONFLICT DO NOTHING;